//
//  ViewController.m
//  SQLiteTutorial
//
//  Created by KH1386 on 5/15/14.
//  Copyright (c) 2014 KH1386. All rights reserved.
//

#import "ViewController.h"
#import "StudentsListControllViewController.h"
#import <sqlite3.h>

@interface ViewController ()

@end

StudentsListControllViewController * tableController=nil;
@implementation ViewController

@synthesize studentNameBox;
@synthesize ageBox;
@synthesize marksBox;
@synthesize nameBox2;
-(NSString *) getDbFilePath
{
    NSString * docsPath= NSSearchPathForDirectoriesInDomains (NSDocumentDirectory, NSUserDomainMask, YES)[0];
    
    return [docsPath stringByAppendingPathComponent:@"studets.db"];
}
- (void)viewDidLoad
{
    [super viewDidLoad];
    [studentNameBox setDelegate:self];
    [nameBox2 setDelegate:self];
    [ageBox setDelegate:self];
    [marksBox setDelegate:self];
    
    tableController = [self.storyboard instantiateViewControllerWithIdentifier:@"StudentsListControllViewController"];
    
    if(![[NSFileManager defaultManager] fileExistsAtPath:[self getDbFilePath]]) //if the file does not exist
    {
        [self createTable:[self getDbFilePath]];
    }
	// Do any additional setup after loading the view, typically from a nib.
}

-(int) createTable:(NSString*) filePath
{
    sqlite3* db = NULL;
    int rc=0;
    
    rc = sqlite3_open_v2([filePath cStringUsingEncoding:NSUTF8StringEncoding], &db, SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE, NULL);
    if (SQLITE_OK != rc)
    {
        sqlite3_close(db);
        NSLog(@"Failed to open db connection");
    }
    else
    {
        char * query ="CREATE TABLE IF NOT EXISTS students ( id INTEGER PRIMARY KEY AUTOINCREMENT, name  TEXT, age INTEGER, marks INTEGER )";
        char * errMsg;
        rc = sqlite3_exec(db, query,NULL,NULL,&errMsg);
        
        if(SQLITE_OK != rc)
        {
            NSLog(@"Failed to create table rc:%d, msg=%s",rc,errMsg);
        }
        
        sqlite3_close(db);
    }
    
    return rc;
    
}
-(int) delete:(NSString*) filePath withName:(NSString*) name
{
    sqlite3* db = NULL;
    int rc=0;
    rc = sqlite3_open_v2([filePath cStringUsingEncoding:NSUTF8StringEncoding], &db, SQLITE_OPEN_READWRITE , NULL);
    if (SQLITE_OK != rc)
    {
        sqlite3_close(db);
        NSLog(@"Failed to open db connection");
    }
    else
    {
        NSString * query  = [NSString
                             stringWithFormat:@"DELETE FROM students where name=\"%@\"",name];
        char * errMsg;
        rc = sqlite3_exec(db, [query UTF8String] ,NULL,NULL,&errMsg);
        if(SQLITE_OK != rc)
        {
            NSLog(@"Failed to delete record  rc:%d, msg=%s",rc,errMsg);
        }
        sqlite3_close(db);
    }

    return  rc;
}
-(int) insert:(NSString *)filePath withName:(NSString *)name age:(NSInteger)age marks:(NSInteger)marks
{
    sqlite3* db = NULL;
    int rc=0;
    rc = sqlite3_open_v2([filePath cStringUsingEncoding:NSUTF8StringEncoding], &db, SQLITE_OPEN_READWRITE , NULL);
    if (SQLITE_OK != rc)
    {
        sqlite3_close(db);
        NSLog(@"Failed to open db connection");
    }
    else
    {
        NSString * query  = [NSString
                                 stringWithFormat:@"INSERT INTO students (name,age,marks) VALUES (\"%@\",%ld,%ld)",name,(long)age,(long)marks];
        char * errMsg;
        rc = sqlite3_exec(db, [query UTF8String] ,NULL,NULL,&errMsg);
        if(SQLITE_OK != rc)
        {
            NSLog(@"Failed to insert record  rc:%d, msg=%s",rc,errMsg);
        }
        sqlite3_close(db);
    }
    return rc;
}
-(int) insertWithoutExec:(NSString *)filePath withName:(NSString *)name age:(NSInteger)age marks:(NSInteger)marks
{
    sqlite3* db = NULL;
    sqlite3_stmt* stmt =NULL;

    int rc=0;
    rc = sqlite3_open_v2([filePath cStringUsingEncoding:NSUTF8StringEncoding], &db, SQLITE_OPEN_READWRITE , NULL);
    if (SQLITE_OK != rc)
    {
        sqlite3_close(db);
        NSLog(@"Failed to open db connection");
    }
    else
    {
        NSString * query  = [NSString
                             stringWithFormat:@"INSERT INTO students (name,age,marks) VALUES (\"%@\",%ld,%ld)",name,(long)age,(long)marks];

        
        rc =sqlite3_prepare_v2(db, [query UTF8String], -1, &stmt, NULL);
        if(rc == SQLITE_OK)
        {
            rc =sqlite3_step(stmt);
            if(rc == SQLITE_DONE) //success
            {
                rc = SQLITE_OK;
            }
            
            sqlite3_finalize(stmt);
        }
        
        sqlite3_close(db);
    }
    return rc;
}


-(NSArray *) getRecords:(NSString*) filePath where:(NSString *)whereStmt
{
    NSMutableArray * students =[[NSMutableArray alloc] init];
    sqlite3* db = NULL;
    sqlite3_stmt* stmt =NULL;
    int rc=0;
    rc = sqlite3_open_v2([filePath UTF8String], &db, SQLITE_OPEN_READONLY , NULL);
    if (SQLITE_OK != rc)
    {
        sqlite3_close(db);
        NSLog(@"Failed to open db connection");
    }
    else
    {
        NSString  * query = @"SELECT * from students";
        if(whereStmt)
        {
            query = [query stringByAppendingFormat:@" WHERE %@",whereStmt];
        }
        
        rc =sqlite3_prepare_v2(db, [query UTF8String], -1, &stmt, NULL);
        if(rc == SQLITE_OK)
        {
            while (sqlite3_step(stmt) == SQLITE_ROW) //get each row in loop
            {
                
                NSString * name =[NSString stringWithUTF8String:(const char *)sqlite3_column_text(stmt, 1)];
                NSInteger age =  sqlite3_column_int(stmt, 2);
                NSInteger marks =  sqlite3_column_int(stmt, 3);

                NSDictionary *student =[NSDictionary dictionaryWithObjectsAndKeys:name,@"name",
                                        [NSNumber numberWithInteger:age],@"age",[NSNumber numberWithInteger:marks], @"marks",nil];
                
                [students addObject:student];
                NSLog(@"name: %@, age=%ld , marks =%ld",name,(long)age,(long)marks);
                
            }
            NSLog(@"Done");
            sqlite3_finalize(stmt);
        }
        else
        {
            NSLog(@"Failed to prepare statement with rc:%d",rc);
        }
        sqlite3_close(db);
    }
    
    return students;
    
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

-(void) showMessage:(NSString*)title withMessage:(NSString*)message
{
    [[[UIAlertView alloc] initWithTitle:title message:message delegate:nil cancelButtonTitle:@"Cancel" otherButtonTitles: nil] show];

}
-(IBAction) addRecord:(id)sender
{
    NSString * name = studentNameBox.text;
    NSInteger age = [ageBox.text integerValue];
    NSInteger marks = [marksBox.text integerValue];
    
    if(studentNameBox.text.length ==0 || ageBox.text.length ==0 || marksBox.text.length ==0)
    {
        [self showMessage:@"ERROR" withMessage:@"Enter Valid Name/Age/Marks"];
    }
    else
    {
       int rc= [self insert:[self getDbFilePath] withName:name age:age marks:marks];
        if(rc != SQLITE_OK)
        {
            [self showMessage:@"ERROR" withMessage:@"Failed to insert record"];
        }
        else
            [self showMessage:@"SUCCESS" withMessage:@"Record is added"];
    }
    
}
-(IBAction) showRecords:(id)sender
{
    NSArray * students = [self getRecords:[self getDbFilePath] where:nil];
    [tableController setStudents:students];
    [self.navigationController pushViewController:tableController animated:YES];
}
-(IBAction) searchByName:(id)sender
{
    NSString * name = nameBox2.text;
    NSArray * students =  [self getRecords:[self getDbFilePath] where:[NSString stringWithFormat:@"name=\"%@\"",name]];
    [tableController setStudents:students];
    
    [self.navigationController pushViewController:tableController animated:YES];
    
    
}
-(IBAction) deleteByName:(id)sender
{
    NSString * name = nameBox2.text;
    
    int rc = [self delete:[self getDbFilePath] withName:name];
    if(rc != SQLITE_OK)
    {
        [self showMessage:@"ERROR" withMessage:@"Failed to delete record"];
    }
    else
        [self showMessage:@"SUCCESS" withMessage:@"Record is deleted"];


}
- (BOOL)textFieldShouldReturn:(UITextField *)textField;
{
    [textField resignFirstResponder];

    return YES;
}

@end
